/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef STEREOWATCHDOGCONTROLLER_H
#define STEREOWATCHDOGCONTROLLER_H

#include <Controller.h>
#include "../../NomadModules/src/drivers/icp/icpcon7217/ICPcon7217Driver.h"
#include "../../NomadModules/src/drivers/icp/icpcon7217/ICPcon7217ChannelDriver.h"
#include <boost/thread.hpp>


namespace stereo_watchdog {


/*!
 * \class StereoWatchdogController
 */

class StereoWatchdogController: public ExperimentController{

public:

	//! Type of controller
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 *
	 * It is the default constructor called by the creator class ExperimentControllerCreator.
	 *
	 * \param[in] name the name of the controller
	 */
	StereoWatchdogController(const std::string& name);

	/*!
	 * \brief Copy constructor
	 *
	 * \param[in] controller the controller to copy
	 */
	StereoWatchdogController(const StereoWatchdogController& controller);

	/*!
	 * \brief Destructor
	 */
	virtual ~StereoWatchdogController();

	/*!
	 * \brief Method called before changing the property value
	 *
	 * This method is called after setting configuration during the creation of controller.
	 */
	virtual void postConfiguration();

	/*!
	 * Properties
	 */



protected:
		Property<int32> d0;
		Property<bool> safe;
		Property<float64> temperaturemax;
		Property<float64> temperature;
		Property<bool> level1;
		Property<bool> level2;

		DriverPtr<icpcon7217::ICPcon7217Driver> m_Driver; //! The ICPconDriver itself
	    DriverPtr<icpcon7217::ICPcon7217ChannelDriver> m_TemperatureDriver; //! The driver linked to the controller
	    DriverPtr<icpcon7217::ICPcon7217ChannelDriver> m_Level1Driver; //! The driver linked to the controller
	    DriverPtr<icpcon7217::ICPcon7217ChannelDriver> m_Level2Driver; //! The driver linked to the controller



	    /*!
	    * \brief start
	     *
	     */
	    virtual void start(){};



private:
	    boost::thread * m_serverThread;
	    void Loop();

	    virtual void updateTemperature();
	    virtual void updateLevel1();
	    virtual void updateLevel2();

/*!
	 * \brief update the progression property from driver one
	 */
	void updateProgression();
};
}
#endif // STEREOWATCHDOGCONTROLLER_H
